import types

def xml_reference_helper(ref_list, node, check_map):

    #print "xml_reference_helper: %s" % node.nodeName
    
    if node.nodeType != node.ELEMENT_NODE:
        return

    for child in node.childNodes:
        xml_reference_helper(ref_list, child, check_map)

    name = node.nodeName

    c = check_map.get(name, None)
    if c is not None:
        ref = c(node)
        if ref is not None:
            if type(ref) == types.TupleType or type(ref) == types.ListType:
                ref_list.extend(ref)
            else:
                ref_list.append(ref)


def check_link(node):
    link = node.getAttribute('url')
    return ReferenceInfo(node.get_content(), link)

def check_image(node):
    path = node.getAttribute('path')
    link = node.getAttribute('link')

    # XXX should this be necesary ?
    if path and path[:1]=='/':
        path_info = ReferenceInfo.PATH_ZOPE_ROOT
    else:
        path_info = ReferenceInfo.PATH_CONTAINER
    path_ref = ReferenceInfo(node.get_content(), path,
                             ref_type = ReferenceInfo.REF_INTERN,
                             restriction=ReferenceInfo.RESTRICT_IMAGE,
                             path_info = path_info)
    refs = [ path_ref ]
    if link:
        link_ref = ReferenceInfo(node.get_content(), link)
        refs.append(link_ref)
    return refs


def check_file(node):
    
    # unused (except for the SilvaSampleDocument.SampleDocument ;-)
    path = node.getAttribute('path')

    return ReferenceInfo(node.get_content(), path,
                         ref_type = ReferenceInfo.REF_INTERN,
                         restriction = ReferenceInfo.RESTRICT_FILE,
                         path_info = ReferenceInfo.PATH_SILVA_ROOT)
    

def check_code(node):
    path = node.getAttribute('path')

    return ReferenceInfo(node.get_content(), path,
                         ref_type = ReferenceInfo.REF_INTERN,
                         restriction = ReferenceInfo.RESTRICT_CODE,
                         path_info = ReferenceInfo.PATH_ZOPE_ROOT)

        
def check_external_data(node):
    path = node.getAttribute('path')

    # hm: data source restriction ?
    return ReferenceInfo(node.get_content(), path,
                         ref_type = ReferenceInfo.REF_INTERN,
                         restriction = ReferenceInfo.RESTRICT_EXTSOURCE,
                         path_info = ReferenceInfo.PATH_ZOPE_ROOT)


default_check_map = {
    'link' : check_link,
    'file' : check_file,
    'image': check_image,
    'code' : check_code,
    'externaldata' : check_external_data,
    }
    
