import sys

from xmlrpclib import ServerProxy, Fault

import zLOG

from adapters.reference_info import ReferenceInfo

def get_link_checker(url):
    # late import to break cycles
    import config

    # TODO: we do not need to create hoardes of clients here
    client = LMSClient(config.lms_url, config.lms_client_id)
    lms = LMSWrapper(client, url)
    return lms.doCheck


_lms_state_map={
    'ok': ReferenceInfo.STATUS_OK,
    'unavailable': ReferenceInfo.STATUS_BROKEN,
    'temporary unavailable': ReferenceInfo.STATUS_TIMEOUT,
    # not yet checked ...
    'unknown': ReferenceInfo.STATUS_UNCHECKED
    }

class LMSWrapper:
    " small helper, just to touch the LMSClient as little as possible "
    
    def __init__(self, client, ref):
        self.ref = ref
        self.client = client

    def doCheck(self):
        if self.ref.status != ReferenceInfo.STATUS_UNKNOWN:
            return
        try:
            url = self.ref.get_target()
            self.client.registerLink(url)
            status, reason = self.client.getStatus(url)
            self.ref.status = _lms_state_map.get(status, ReferenceInfo.STATUS_UNCHECKED)
            self.ref.message = self._trim_message(reason)
        except:
            zLOG.LOG('Silva Reference Checker',100,
                     'bad exception while checking url %s' % self.ref.get_target(), '',
                     sys.exc_info())
            self.ref.status = self.ref.STATUS_BROKEN
            self.ref.message = 'link checker failed'
        
    def _trim_message(self, message):
        # ui tweaks : trim the status message from
        # this should maybe not be done here
        message = message.split('.')[0]
        return message


# copy from CMSLinkChecker.database :-/

class LMSClient:
    """client to lms"""

    _exception_mapping = {
        'KeyError': KeyError,
    }

    def __init__(self, url, client_id):
        self.client_id = client_id
        self._server = ServerProxy(url)

    def registerLink(self, url):
        return self._server.registerLink(url, self.client_id)

    def unregisterLink(url):
        return self._server.unregisterLink(url, self.client_id)

    def registerManyLinks(self, urls):
        result = []
        if urls:
            result = self._server.registerManyLinks(urls, self.client_id)
        return result

    def getStatus(self, url):
        try:
            status = self._server.getStatus(url)
        except Fault, f:
            raise self._translate_fault(f)
        return status

    def getClientNotifications(self):
        try:
            return self._server.getClientNotifications(self.client_id)
        except Fault, f:
            raise self._translate_fault(f)

    def setClientNotifications(self, status):
        if isinstance(status, (str, unicode)):
            status = int(status)
        if isinstance(status, int):
            status = bool(status)
        self._server.setClientNotifications(self.client_id, status)


    def checkConnection(self):
        is_ok, error = False, 'unknown'

        try:
            is_ok, error = self._server.checkConnection(self.client_id)
        except Fault, f:
            is_ok, error = False, '%s (%s)' % (f.faultString, f.faultCode)
        except socket.error, e:
            e_code, e_string = e.args
            is_ok, error = False, 'socket.error: %s (%s)' % (e_string, e_code)
        except Exception, e:
            is_ok, error = False, str(e)

        return is_ok, error

    #########
    # private

    def _translate_fault(self, f):
        name, value = f.faultCode, f.faultString

        exception_class = self._exception_mapping.get(name)
        if exception_class is None:
            e = f
        else:
            e = exception_class(value)
            
        return e
        
        

        
    
